# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import (
    report_step_params,
    report_wrong_params,
    report_missing_params,
    report_relationship_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["ReportResource", "AsyncReportResource"]


class ReportResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/StructifyAI/structify-python#accessing-raw-response-data-eg-headers
        """
        return ReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/StructifyAI/structify-python#with_streaming_response
        """
        return ReportResourceWithStreamingResponse(self)

    def missing(
        self,
        *,
        id: str,
        property: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Returns a success message if the report was added successfully Throws an error
        if the property for this entity has already been reported

        Args:
          property: A property that is being reported

          source_url: Correct source URL for the reported entity

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            "/report/entity/missing",
            body=maybe_transform(
                {
                    "id": id,
                    "property": property,
                    "source_url": source_url,
                },
                report_missing_params.ReportMissingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    def relationship(
        self,
        *,
        relationship_type: str,
        source_id: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        target_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Reports a missing relationship between entities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            "/report/relationship/missing",
            body=maybe_transform(
                {
                    "relationship_type": relationship_type,
                    "source_id": source_id,
                    "source_url": source_url,
                    "target_id": target_id,
                },
                report_relationship_params.ReportRelationshipParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    def step(
        self,
        *,
        step_id: str,
        message: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Report a step

        Args:
          message: A short message about why the step is being reported

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            "/report/step",
            body=maybe_transform(
                {
                    "step_id": step_id,
                    "message": message,
                },
                report_step_params.ReportStepParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    def wrong(
        self,
        *,
        id: str,
        property: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Reports a wrong property for an entity

        Args:
          property: A property that is being reported

          source_url: Correct source URL for the reported entity

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return self._post(
            "/report/entity/wrong",
            body=maybe_transform(
                {
                    "id": id,
                    "property": property,
                    "source_url": source_url,
                },
                report_wrong_params.ReportWrongParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncReportResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/StructifyAI/structify-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/StructifyAI/structify-python#with_streaming_response
        """
        return AsyncReportResourceWithStreamingResponse(self)

    async def missing(
        self,
        *,
        id: str,
        property: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Returns a success message if the report was added successfully Throws an error
        if the property for this entity has already been reported

        Args:
          property: A property that is being reported

          source_url: Correct source URL for the reported entity

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            "/report/entity/missing",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "property": property,
                    "source_url": source_url,
                },
                report_missing_params.ReportMissingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    async def relationship(
        self,
        *,
        relationship_type: str,
        source_id: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        target_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Reports a missing relationship between entities

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            "/report/relationship/missing",
            body=await async_maybe_transform(
                {
                    "relationship_type": relationship_type,
                    "source_id": source_id,
                    "source_url": source_url,
                    "target_id": target_id,
                },
                report_relationship_params.ReportRelationshipParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    async def step(
        self,
        *,
        step_id: str,
        message: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Report a step

        Args:
          message: A short message about why the step is being reported

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            "/report/step",
            body=await async_maybe_transform(
                {
                    "step_id": step_id,
                    "message": message,
                },
                report_step_params.ReportStepParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    async def wrong(
        self,
        *,
        id: str,
        property: Optional[str] | NotGiven = NOT_GIVEN,
        source_url: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> str:
        """
        Reports a wrong property for an entity

        Args:
          property: A property that is being reported

          source_url: Correct source URL for the reported entity

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "text/plain", **(extra_headers or {})}
        return await self._post(
            "/report/entity/wrong",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "property": property,
                    "source_url": source_url,
                },
                report_wrong_params.ReportWrongParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class ReportResourceWithRawResponse:
    def __init__(self, report: ReportResource) -> None:
        self._report = report

        self.missing = to_raw_response_wrapper(
            report.missing,
        )
        self.relationship = to_raw_response_wrapper(
            report.relationship,
        )
        self.step = to_raw_response_wrapper(
            report.step,
        )
        self.wrong = to_raw_response_wrapper(
            report.wrong,
        )


class AsyncReportResourceWithRawResponse:
    def __init__(self, report: AsyncReportResource) -> None:
        self._report = report

        self.missing = async_to_raw_response_wrapper(
            report.missing,
        )
        self.relationship = async_to_raw_response_wrapper(
            report.relationship,
        )
        self.step = async_to_raw_response_wrapper(
            report.step,
        )
        self.wrong = async_to_raw_response_wrapper(
            report.wrong,
        )


class ReportResourceWithStreamingResponse:
    def __init__(self, report: ReportResource) -> None:
        self._report = report

        self.missing = to_streamed_response_wrapper(
            report.missing,
        )
        self.relationship = to_streamed_response_wrapper(
            report.relationship,
        )
        self.step = to_streamed_response_wrapper(
            report.step,
        )
        self.wrong = to_streamed_response_wrapper(
            report.wrong,
        )


class AsyncReportResourceWithStreamingResponse:
    def __init__(self, report: AsyncReportResource) -> None:
        self._report = report

        self.missing = async_to_streamed_response_wrapper(
            report.missing,
        )
        self.relationship = async_to_streamed_response_wrapper(
            report.relationship,
        )
        self.step = async_to_streamed_response_wrapper(
            report.step,
        )
        self.wrong = async_to_streamed_response_wrapper(
            report.wrong,
        )
